/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/ 

#include "includes.h"

#define FIFO_SIZE 40

typedef struct
{
    int push_indx;
    int pop_indx;
    float buffer[100];
} FIFO;

int loop = 1, keypress;
float scale_factor;
HDC hdc;
FILE *inifile;
BITMAP *scrat, *frame, *pith_tape,*bank_niddle,*sharik,*trubka;
float bank_angle = 0;
float pitch = 0.0;
float sharik_pos = 0;

int initialise_window( void );
int connect_to_sim( void );
int update_display( void );
int create_bitmaps( void );

int act_on_keypress( void );
int close_program( void );
void exit_hook (void );


static void fifo_init(FIFO *fifo)
{
    fifo->push_indx = 0;
    fifo->pop_indx = 0;
}

static int fifo_push(FIFO *fifo, float data,int fifo_size)
{
    int indx_tmp;
    indx_tmp = fifo->push_indx + 1;
    indx_tmp = indx_tmp % fifo_size;

    if (indx_tmp == fifo->pop_indx)
    {
        return 0;
    }

    fifo->buffer[fifo->push_indx] = data;
    fifo->push_indx = indx_tmp;
    return 1;
}

static int fifo_pop(FIFO *fifo, float *data,int fifo_size)
{
    int indx_tmp;
    if (fifo->push_indx == fifo->pop_indx)
    {
        return 0;
    }
    indx_tmp = fifo->pop_indx + 1;
    indx_tmp = indx_tmp % fifo_size;
    *data = fifo->buffer[fifo->pop_indx];
    fifo->pop_indx = indx_tmp;
    return 1;
}


float moving_average_filter(float cur)
{
    float pre;
    static float ref_sum = 0.0;
    static FIFO f;
    static int flag = 0;

    if(flag==0)
    {
        fifo_init(&f);
        flag = 1;
    }

    ref_sum += cur;
    if(!fifo_push(&f,cur, FIFO_SIZE))
    {
        fifo_pop(&f,&pre, FIFO_SIZE);
        fifo_push(&f,cur, FIFO_SIZE);
        ref_sum -= pre;
    }
    else
        return 0.0;
    return ref_sum/((FIFO_SIZE-1)*1.0);
}

float moving_average_filter_pitch(float cur)
{
    float pre;
    static float ref_sum = 0.0;
    static FIFO f;
    static int flag = 0;

    if(flag==0)
    {
        fifo_init(&f);
        flag = 1;
    }

    ref_sum += cur;
    if(!fifo_push(&f,cur, FIFO_SIZE))
    {
        fifo_pop(&f,&pre, FIFO_SIZE);
        fifo_push(&f,cur, FIFO_SIZE);
        ref_sum -= pre;
    }
    else
        return 0.0;
    return ref_sum/((FIFO_SIZE-1)*1.0);
}


int main(int argc, char *argv[] )
{

    STARTUPINFO cif;
    PROCESS_INFORMATION pi;
    load_config();

    if(0 == strcmp(config.console,"off") && argc == 1)
    {
        ZeroMemory(&cif,sizeof(STARTUPINFO));
        if(CreateProcess(NULL,"agd.exe AS_DAEMON",
                         NULL,NULL,FALSE,CREATE_NO_WINDOW,NULL,NULL,&cif,&pi))

        {
            CloseHandle( pi.hProcess );
            CloseHandle( pi.hThread );
            exit(0);
        }
    }

    if(config.uplink_port)
        server_start(config.uplink_port);


    if(config.downlink_port)
    {
        udp_sender_start(config.downlink_port, config.downlink_ip);
    }


    initialise_window();

    if( create_bitmaps() == -1 )
    {
        textprintf_centre(screen, font, config.size / 2, config.size * 0.15, 65535,  "*** Error ***" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.40, 65535,  "No Graphics files" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.65, 65535,  "Press Key to Exit" );
        readkey();
        close_program();
        return( -1 );
    }

    update_display();

    while( loop == 1 )
    {

        while( keypressed() )
        {
            act_on_keypress();
            update_display();
        }
        clear_keybuf();
        update_display();
    }

    save_config();
    close_program();
    return (0);
}
END_OF_MAIN()

int initialise_window( void )
{
    hdc = GetDC(NULL);
    int screen_colour_depth = GetDeviceCaps (hdc, BITSPIXEL );

    if (allegro_init() != 0) return 1; //Initialise Allegro
    install_keyboard();

    set_color_depth(screen_colour_depth);


    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0) != 0)
    {
        if (set_gfx_mode(GFX_SAFE, 800, 600, 0, 0) != 0)
        {
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
            return 1;
        }
    }

    SetWindowPos(win_get_window(), HWND_TOPMOST,
                 config.x_pos,
                 config.y_pos,
                 config.size+6,
                 config.size+25,
                 SWP_ASYNCWINDOWPOS);

    set_display_switch_mode(SWITCH_BACKGROUND);
    scale_factor = ((float)config.size / 400);

    set_window_close_hook(exit_hook);
    return( 0 );
}



int update_display( void )

{
    int size = config.size;


    pivot_scaled_sprite(scrat, pith_tape, size / 2, (size / 2),128, 512+pitch,
                        ftofix(0) , ftofix(scale_factor));



    pivot_scaled_sprite(scrat, trubka, size / 2, (0 / 2),100, 0,
                        ftofix(0) , ftofix(scale_factor));

    pivot_scaled_sprite(scrat, sharik, size / 2, (0 / 2),15, 0,
                        ftofix(sharik_pos) , ftofix(scale_factor));

    pivot_scaled_sprite(scrat, frame, size / 2, (size / 2),200, 200,
                        ftofix(0) ,ftofix(scale_factor));


    pivot_scaled_sprite(scrat, bank_niddle, size / 2, (size / 2),200, 200,
                        ftofix(bank_angle) , ftofix(scale_factor));

    if(console_flag)
        printf("bank: %f   pitch: %f sideG: %f\n",
           bank_angle,
           pitch,
           sharik_pos
          );

    blit(scrat, screen, 0, 0, 0, 0, size, size);
    Sleep(20);
    return( 0 );
}

int create_bitmaps( void )
{
    int size = config.size;
    scrat = create_bitmap(size, size);
    if( (pith_tape = load_bitmap( "..\\Resources\\agd\\ag_tape.bmp", pith_tape )) == NULL ) return( -1 );
    if( (bank_niddle = load_bitmap( "..\\Resources\\agd\\siluet.bmp", bank_niddle )) == NULL ) return( -1 );
    if( (frame = load_bitmap( "..\\Resources\\agd\\agd-frame.bmp", frame )) == NULL ) return( -1 );
    if( (sharik = load_bitmap( "..\\Resources\\agd\\sharik.bmp", sharik )) == NULL ) return( -1 );
    if( (trubka = load_bitmap( "..\\Resources\\agd\\trubka.bmp", trubka )) == NULL ) return( -1 );
    return( 0 );
}

int act_on_keypress( void )
{
    RECT rect;
    if (GetWindowRect(win_get_window(), &rect))
    {
        config.x_pos = (int)rect.left;
        config.y_pos = (int)rect.top;
    }
    keypress = readkey();
    switch (keypress)
    {

    case 6778: //
        bank_angle -= 1;
        break;

    case 6264: //x
        bank_angle += 1;
        break;



    case 867: //c
        sharik_pos--;
        break;


    case 5750: //v
        sharik_pos++;
        break;


    case 4465 : //q
        pitch++;
        break;

    case 353: //a
        pitch--;
        break;

    case 6007 : //w

        break;

    case 1381: //e

        break;


    case 2409:         //i pressed ( increase size )
        if ( config.size < 600 ) config.size+=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     HWND_TOPMOST,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);

        scale_factor = ((float)config.size / 400);
        break;

    case 1124:         //d pressed ( decrease size )
        if( config.size > 142 ) config.size-=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(),
                     0,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);
        scale_factor = ((float)config.size / 400);
        break;
    }
    return( 0 );
}
int close_program( void )
{
    allegro_exit();
    return( 0 );
}
void exit_hook( void )
{
    loop = 0;
}

void cb_on_data(int block_id, float *data)
{
    int i;

    switch(block_id)
    {
    case 4:
        sharik_pos = moving_average_filter(data[6]/0.2*8.0);
        if(sharik_pos>8.0) sharik_pos = 8.0;
        if(sharik_pos<-8.0) sharik_pos = -8.0;
        break;


    case 17:
        pitch = (-data[0]*321.0/80.0);
        bank_angle = data[1]*33.0/45.0;
        break;

    }

}








