/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this
  notice you can do whatever you want with this stuff.
  If we meet some day, and you think this stuff is worth it,
  you can buy me a beer in return.
----------------------------------------------------------*/

#include "includes.h"

CONFIG config;
int console_flag = 0;


typedef struct
{
    const char *group;
    const char *key;
    void *var;
    int size;
    int type;
} INIT_STRUCT;

enum
{
    INT_VAL=0,
    LINE

};

#define ADD_ITEM_INT(a,b,c)  {a,b,&config.c,sizeof(config.c),INT_VAL}
#define ADD_ITEM_LINE(a,b,c)  {a,b,config.c,sizeof(config.c),LINE}


const INIT_STRUCT init_struct[] =
{

    ADD_ITEM_INT("gui",          "x_pos",   x_pos),
    ADD_ITEM_INT("gui",          "y_pos",   y_pos),
    ADD_ITEM_INT("gui",          "size",    size),
    ADD_ITEM_INT("network",      "uplink_port",  uplink_port),
    ADD_ITEM_LINE("network",     "downlink_ip",    downlink_ip),
    ADD_ITEM_INT("network",      "downlink_port",  downlink_port),
    ADD_ITEM_LINE("misc",         "console",  console),
    {NULL,NULL,NULL,-1}
};

static int config_cb(void* user, const char* section, const char* name,
                     const char* value)
{
    int i;

#define MATCH(s, n) strcmp(section, s) == 0 && strcmp(name, n) == 0

    for(i=0; init_struct[i].size!=-1; i++)
    {
        if(MATCH(init_struct[i].group, init_struct[i].key))
        {
            if(init_struct[i].type == INT_VAL)
            {
                *((int*)init_struct[i].var) = atoi(value);
            }
            else  if(init_struct[i].type == LINE)
            {
                strncpy((char*)init_struct[i].var,value,init_struct[i].size);
            }

        }
    }
    return 1;
}



int load_config(void)
{
    if(ini_parse(INI_FILE_NAME, config_cb, &config) < 0)
    {
        config.x_pos = 0;
        config.y_pos = 0;
        config.size = 300;
        strcpy(config.downlink_ip,"127.0.0.1");
        config.uplink_port = 49077;
        config.downlink_port = 0;
        strcpy(config.console,"on");
        save_config();
    }

    if(0 == strcmp(config.console,"on"))
        console_flag = 1;
    else
        console_flag = 0;

    return 0;
}

int save_config(void)
{
    FILE *fp = fopen(INI_FILE_NAME,"wb");
    if(fp)
    {
        fprintf(fp,"[gui]\r\n");
        fprintf(fp,"x_pos=%ld\r\n",   config.x_pos);
        fprintf(fp,"y_pos=%ld\r\n",   config.y_pos);
        fprintf(fp,"size=%ld\r\n",    config.size);
        fprintf(fp,"\r\n[network]\r\n");
        fprintf(fp,"uplink_port=%ld\r\n",  config.uplink_port);
        fprintf(fp,"downlink_ip=%s\r\n",    config.downlink_ip);
        fprintf(fp,"downlink_port=%ld\r\n",  config.downlink_port);
        fprintf(fp,"\r\n[misc]\r\n");
        fprintf(fp,"console=%s\r\n",    config.console);
        fclose(fp);
    }

}

int show_config(void)
{
    printf("[gui]\n");
    printf("x_pos=%ld\n",   config.x_pos);
    printf("y_pos=%ld\n",   config.y_pos);
    printf("size=%ld\n",    config.size);
    printf("\n[network]\n");
    printf("uplink_port=%ld\n",  config.uplink_port);
    printf("downlink_ip=%s\n",    config.downlink_ip);
    printf("downlink_port=%ld\n",  config.downlink_port);
    printf("\n[misc]\n");
    printf("console=%s\n", config.console);
}

