/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/ 

#include "includes.h"

int loop = 1, keypress;
float scale_factor;
HDC hdc;
FILE *inifile;
BITMAP *scrat, *background, *pointer,*index,*limb,*horiz,*vert,*heading;
PALETTE palette;
float  crs = 0;
float gs = 0;
float loc = 0;
float hdg = 0;
int nav_src = 0;
int nav_flag = 0;
int mouse_flag = 0;

int initialise_window( void );
int connect_to_sim( void );
int update_display( void );
int create_bitmaps( void );
int get_knots( void );
int act_on_keypress( void );
int close_program( void );
void exit_hook (void );


int main(int argc, char *argv[] )
{
    STARTUPINFO cif;
    PROCESS_INFORMATION pi;
    load_config();

    if(0 == strcmp(config.console,"off") && argc == 1)
    {

        ZeroMemory(&cif,sizeof(STARTUPINFO));
        if(CreateProcess(NULL,"kppm.exe AS_DAEMON",
                         NULL,NULL,FALSE,CREATE_NO_WINDOW,NULL,NULL,&cif,&pi))

        {
            CloseHandle( pi.hProcess );
            CloseHandle( pi.hThread );
            exit(0);
        }
    }

    if(config.uplink_port)
        server_start(config.uplink_port);


    if(config.downlink_port)
    {
        udp_sender_start(config.downlink_port, config.downlink_ip);
    }


    initialise_window();

    printf("mouse buttons number = %d\n",install_mouse());
    show_os_cursor(2);
    if(mouse_needs_poll())
        printf("the current mouse driver is operating in polling mode\n");


    if( create_bitmaps() == -1 )
    {
        textprintf_centre(screen, font, config.size / 2, config.size * 0.15, 65535,  "*** Error ***" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.40, 65535,  "No Graphics files" );
        textprintf_centre(screen, font, config.size / 2, config.size * 0.65, 65535,  "Press Key to Exit" );
        readkey();
        close_program();
        return( -1 );
    }
    update_display();

    while( loop == 1 )
    {

        while( keypressed() )
        {
            act_on_keypress();
            update_display();
        }
        clear_keybuf();
        update_display();
    }
    save_config();
    close_program();
    return (0);
}

END_OF_MAIN()

int initialise_window( void )
{
    hdc = GetDC(NULL);
    int screen_colour_depth = GetDeviceCaps (hdc, BITSPIXEL );

    if (allegro_init() != 0) return 1; //Initialise Allegro
    install_keyboard();

    set_color_depth(screen_colour_depth);


    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0) != 0)
    {
        if (set_gfx_mode(GFX_SAFE, 800, 600, 0, 0) != 0)
        {
            set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
            allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
            return 1;
        }
    }

    SetWindowPos(win_get_window(), HWND_TOPMOST,
                 config.x_pos,
                 config.y_pos,
                 config.size+6,
                 config.size+25, SWP_ASYNCWINDOWPOS);
    set_display_switch_mode(SWITCH_BACKGROUND);
    scale_factor = ((float)config.size / 400);

    set_window_close_hook(exit_hook);
    return( 0 );
}


int mouse_action(int mouse_state)
{
    if (mouse_state & 1)
    {
        nav_src++;
        if(nav_src>2)
            nav_src = 0;
        return 1;

    }

    return 0;
}



int update_display( void )
{
    int size = config.size;
    int mouse_state = mouse_b;
    stretch_blit( background, scrat, 0,0,400,400,0,0,size,size);
    pivot_scaled_sprite(scrat, vert, size / 2, (size / 2), (12-6) + round(loc), 102, ftofix(0) , ftofix(scale_factor));
    pivot_scaled_sprite(scrat, horiz, size / 2, (size / 2), 102, (12-5)+round(gs), ftofix(0) , ftofix(scale_factor));

    rotate_scaled_sprite(scrat, limb, 00, 00, ftofix(-76.0 - crs/1.40625) , ftofix(scale_factor));
    pivot_scaled_sprite(scrat, heading, size / 2, (size / 2), 17, 168, ftofix(hdg/1.40625) , ftofix(scale_factor));
    rotate_scaled_sprite(scrat,
                         index,
                         size / 2 - round(5.0*scale_factor) ,
                         round(18.0*scale_factor),
                         ftofix(0) ,
                         ftofix(scale_factor));


    if(mouse_state & 3 && mouse_flag)
    {

    }
    else if (!(mouse_state & 3) && mouse_flag)
    {
        mouse_flag = 0;
    }


    if(mouse_flag == 0)
    {
        mouse_flag = mouse_action(mouse_state);
    }


    const char *nav_src_name[] =
    {
        "NAV1",
        "NAV2",
        "GPS"
    };

    const char *nav_flag_name[] =
    {
        "--- ",
        "TO  ",
        "FROM",
        "    "
    };
    textprintf_ex(scrat, font, 5, 5, makecol(000, 255, 000),-1, nav_src_name[nav_src]);
    textprintf_ex(scrat, font, size-40, 5, makecol(000, 255, 255),-1, nav_flag_name[nav_flag]);


    blit(scrat, screen, 0, 0, 0, 0, size, size);
    Sleep(50);
    return( 0 );
}

int create_bitmaps( void )
{
    scrat = create_bitmap(config.size, config.size);
    if( (background = load_bitmap( "..\\Resources\\kppm\\kppm_bg.bmp", palette )) == NULL ) return( -1 );
    if( (index = load_bitmap( "..\\Resources\\kppm\\index.bmp", index )) == NULL ) return( -1 );
    if( (limb = load_bitmap( "..\\Resources\\kppm\\kppm_limb.bmp", limb )) == NULL ) return( -1 );
    if( (horiz = load_bitmap( "..\\Resources\\kppm\\horiz_niddle.bmp", horiz )) == NULL ) return( -1 );
    if( (vert = load_bitmap( "..\\Resources\\kppm\\vert_niddle.bmp", vert )) == NULL ) return( -1 );
    if( (heading = load_bitmap( "..\\Resources\\kppm\\kppm_niddle.bmp", heading )) == NULL ) return( -1 );
    return( 0 );
}



int act_on_keypress( void )
{
    RECT rect;
    if (GetWindowRect(win_get_window(), &rect))
    {
        config.x_pos = (int)rect.left;
        config.y_pos = (int)rect.top;
    }
    keypress = readkey();
    switch (keypress)
    {
    case 6778: //z
        crs+=1.0;
        break;

    case 6264: //x
        crs-=1.0;
        break;



    case 867: //c
        hdg-=1.0;
        break;


    case 5750: //v
        hdg+=1.0;
        break;


    case 4465 : //q
        gs+=1.0;
        break;

    case 353: //a
        gs-=1.0;
        break;

    case 6007 : //w
        loc+=1.0;
        break;

    case 1381: //e
        loc-=1.0;
        break;


    case 2409:         //i pressed ( increase size )
        if ( config.size < 600 ) config.size+=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(), HWND_TOPMOST,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);
        scale_factor = ((float)config.size / 400);
        break;

    case 1124:         //d pressed ( decrease size )
        if( config.size > 142 ) config.size-=1;
        destroy_bitmap( scrat );
        scrat = create_bitmap(config.size, config.size);
        SetWindowPos(win_get_window(), 0,
                     config.x_pos,
                     config.y_pos,
                     config.size+6,
                     config.size+25,
                     SWP_NOZORDER);
        scale_factor = ((float)config.size / 400);
        break;
    }
    return( 0 );
}
int close_program( void )
{
    allegro_exit();
    return( 0 );
}
void exit_hook( void )
{
    loop = 0;
}

void cb_on_data(int block_id, float *data)
{
    int i;
    switch(block_id)
    {
    case 118:
        crs = data[1];
        break;

    case 17:
        hdg = data[3]-crs;
        break;

    case 103:

        if(nav_src == 2)
        {
            loc = -30.4* data[6];
            gs = -30.4* data[7];
        }

        break;

    case 98:
        switch(nav_src)
        {
        case 0:
            nav_flag = data[2];
            break;

        case 1:
            nav_flag = data[2];
            break;

        case 2:
            nav_flag  = 3;
            break;

        }

        break;

    case 99:

        if(nav_src == 0)
        {
            loc = -30.4* data[5];
            gs = -30.4* data[6];
        }

        break;

    case 100:

        if(nav_src == 1)
        {
            loc = -30.4* data[5];
            gs = -30.4* data[6];
        }

        break;
    }
}








